#include "ruby.h"
#include <stdint.h>
#include "fuzzy.h"

static VALUE ssdeep_compare(VALUE module, VALUE h1, VALUE h2)
{
  int n;

  Check_Type(h1, T_STRING);
  Check_Type(h2, T_STRING);

  n = fuzzy_compare(RSTRING(h1)->ptr, RSTRING(h2)->ptr);
  if (n == -1)
    rb_raise(rb_eRuntimeError, "Error while comparing hashes");
  return (INT2FIX(n));
}

static VALUE ssdeep_hash_buf(VALUE module, VALUE buf)
{
  char hash[FUZZY_MAX_RESULT];
  
  Check_Type(buf, T_STRING);
  
  if (fuzzy_hash_buf(RSTRING(buf)->ptr, RSTRING(buf)->len, hash))
    rb_raise(rb_eRuntimeError, "Error while fuzzy hashing");

  return rb_str_new2(hash);
}

static VALUE ssdeep_hash_filename(VALUE module, VALUE path)
{
  char hash[FUZZY_MAX_RESULT];
  
  Check_Type(path, T_STRING);
  
  if (fuzzy_hash_filename(RSTRING(path)->ptr, hash))
    rb_raise(rb_eRuntimeError, "Error while fuzzy hashing");;

  return rb_str_new2(hash);
}

void Init_ssdeep()
{
  VALUE mSsdeep;
  mSsdeep = rb_define_module("Ssdeep");
  rb_define_module_function(mSsdeep, "fuzzy_compare", ssdeep_compare, 2);
  rb_define_module_function(mSsdeep, "fuzzy_hash_buf", ssdeep_hash_buf, 1);
  rb_define_module_function(mSsdeep, "fuzzy_hash_filename", ssdeep_hash_filename, 1);
}
